@@ -38,14 +38,19 @@ void 	     gnome_vfs_mime_shutdown 				(void);
 
 /* do not free any of the returned gchar* values */
 const char  *gnome_vfs_mime_type_from_name			(const gchar* filename);
-const char  *gnome_vfs_mime_type_from_name_or_default	(const gchar *filename,
+const char  *gnome_vfs_mime_type_from_name_or_default	        (const gchar *filename,
 								 const gchar *defaultv);
 
 const char  *gnome_vfs_get_mime_type				(GnomeVFSURI *uri);
+/* FIXME: This function should be named more clearly, maybe
+   to show that it gets the mime type only using the uri information. */
 const char  *gnome_vfs_get_mime_type_from_uri			(GnomeVFSURI *uri);
+ 
+/* FIXME: These two functions should be deprecated after Gnome 1.4.  */
 const char  *gnome_vfs_get_mime_type_from_file_data		(GnomeVFSURI *uri);
 const char  *gnome_vfs_get_mime_type_for_data			(gconstpointer data,
                                                                  int data_size);
+
 const char  *gnome_vfs_get_file_mime_type 			(const char *path, 
 								 const struct stat *optional_stat_info,
 								 gboolean suffix_only);
