@@ -156,7 +156,7 @@ typedef struct GnomeVFSURI {
 	/* Text for the element: eg. some/path/name.  */
 	gchar *text;
 
-	/* Text for the element: eg. some/path/name.  */
+	/* Text for uri fragment: eg, #anchor  */
 	gchar *fragment_id;
 	
 	/* Method string: eg. `gzip', `tar', `http'.  This is necessary as
@@ -310,25 +310,6 @@ typedef enum {
 	GNOME_VFS_DIRECTORY_KIND_TRASH = 1001
 } GnomeVFSFindDirectoryKind;
 
-typedef struct GnomeVFSDirectoryList GnomeVFSDirectoryList;
-
-typedef gpointer GnomeVFSDirectoryListPosition;
-
-#define GNOME_VFS_DIRECTORY_LIST_POSITION_NONE NULL
-
-typedef enum {
-	GNOME_VFS_DIRECTORY_SORT_NONE,
-	GNOME_VFS_DIRECTORY_SORT_DIRECTORYFIRST,
-	GNOME_VFS_DIRECTORY_SORT_BYNAME,
-	GNOME_VFS_DIRECTORY_SORT_BYNAME_IGNORECASE,
-	GNOME_VFS_DIRECTORY_SORT_BYSIZE,
-	GNOME_VFS_DIRECTORY_SORT_BYBLOCKCOUNT,
-	GNOME_VFS_DIRECTORY_SORT_BYATIME,
-	GNOME_VFS_DIRECTORY_SORT_BYMTIME,
-	GNOME_VFS_DIRECTORY_SORT_BYCTIME,
-	GNOME_VFS_DIRECTORY_SORT_BYMIMETYPE
-} GnomeVFSDirectorySortRule;
-
 typedef enum {
 	GNOME_VFS_DIRECTORY_FILTER_NONE,
 	GNOME_VFS_DIRECTORY_FILTER_SHELLPATTERN,
@@ -365,9 +346,6 @@ typedef struct GnomeVFSDirectoryFilter GnomeVFSDirectoryFilter;
 
 typedef gboolean (* GnomeVFSDirectoryFilterFunc) (const GnomeVFSFileInfo *info,
 						  gpointer data);
-typedef gint     (* GnomeVFSDirectorySortFunc)   (const GnomeVFSFileInfo *a,
-						  const GnomeVFSFileInfo *b,
-						  gpointer data);
 typedef gboolean (* GnomeVFSDirectoryVisitFunc)	 (const gchar *rel_path,
 						  GnomeVFSFileInfo *info,
 						  gboolean recursing_will_loop,
@@ -615,7 +593,7 @@ typedef void	(* GnomeVFSAsyncSetFileInfoCallback)
 typedef void	(* GnomeVFSAsyncDirectoryLoadCallback)
 						(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSResult result,
-						 GnomeVFSDirectoryList *list,
+						 GList *list,
 						 guint entries_read,
 						 gpointer callback_data);
 
@@ -639,4 +617,25 @@ typedef void    (* GnomeVFSAsyncFindDirectoryCallback)
 typedef void    (* GnomeVFSStatusCallback)      (const gchar *message,
 						 gpointer     callback_data);
 
+/* VFS Transform */
+typedef struct GnomeVFSTransform GnomeVFSTransform;
+typedef struct GnomeVFSContext GnomeVFSContext;
+
+typedef GnomeVFSTransform * (* GnomeVFSTransformInitFunc)(const char *method_name, const char *config_args);
+
+typedef GnomeVFSResult (* GnomeVFSTransformFunc) (GnomeVFSTransform *transform,
+						  const gchar *old_uri,
+						  gchar **new_uri,
+						  GnomeVFSContext *context);
+
+struct GnomeVFSTransform {
+	GnomeVFSTransformFunc transform;
+};
+
+typedef struct GnomeVFSMessageCallbacks GnomeVFSMessageCallbacks;
+typedef struct GnomeVFSCancellation GnomeVFSCancellation;
+typedef gpointer GnomeVFSMethodHandle;
+typedef struct GnomeVFSIOBuf GnomeVFSIOBuf;
+typedef struct GnomeVFSInetConnection GnomeVFSInetConnection;
+
 #endif /* _GNOME_VFS_TYPES_H */
